//-----------------------------------------------------------------------------
// MK3_2560_SPI_SLAVE_MSS
// Version 1.11
// 02.11.2011 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer das myAVR MK3 Board
// mit ATMega-2560 @ 16MHz
// SPI-SLAVE-Funktion (Multi-Slave-System)
// SPI-Kommunikation zwischen einem Master und bis zu 8 Slaves
// Hinweis : es gibt auch einen Treiber fuer ein Single-Slave-System (SSS)
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_MK3_2560/"
// liegen und mit #include <UB_MK3_2560/MK3_2560_SPI_SLAVE_MSS.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt : Port-B (Bit0 bis Bit3)
// Benutzt : MK3_2560_SPI_MSS.h
//           
//-----------------------------------------------------------------------------
// Verbindungen :
// SS vom Slave muss zum (passenden) Select-Pin des Masters
//   (z.B. SS von Slave-2 muss zum Select-2 vom Master)
// SCK vom Slave muss zum SCK-Pin des Masters
// MOSI vom Slave muss zum MOSI-Pin des Masters
// MISO vom Slave muss zum MISO-Pin vom Master
//-----------------------------------------------------------------------------
#include  "UB_MK3_2560/MK3_2560_SPI_MSS.h"
#include <avr/interrupt.h>


//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "MK3_INIT_SPI_SLAVE" starten
//   -> die komplette SPI-Kommunikation laeuft im Hintergrund per Interrupt ab
// 02. Funktion "MK3_SPIS_AKTIVITY" prueft ob es SPI aktivitaet gab
//   02a : falls Ja -> "MK3_SPIS_GET_ID" uebergibt den Status (fehler-nr oder cmd)
// 03. Funktion "MK3_SPIS_WR8_RAM" schreibt ein Byte ins eigene RAM
// 04. Funktion "MK3_SPIS_RD8_RAM" liest ein Byte aus eigenem RAM
// 05. Funktion "MK3_SPIS_WR16_RAM" schreibt ein INT ins eigene RAM
// 06. Funktion "MK3_SPIS_RD16_RAM" liest ein INT aus eigenem RAM
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktioen
//-----------------------------------------------------------------------------


// Rueckgabewerte von "MK3_SPIS_AKTIVITY"
typedef enum {
  KOM_NOTHING  =0,   // es hat keine neue Kommunikation stattgefunden
  KOM_CMD_EXE,       // ein gueltiges Kommando wurde ausgefuehrt
  KOM_CMD_ERR        // es gab einen Kommando fehler
}SPI_KOM_STATUS_t;


// Rueckgabewerte von "MK3_SPIS_GET_ID"
typedef enum {
  NO_SPI_KOM   =0,   // es hat keine neue Kommunikation stattgefunden
  EXE_CHK_SLAVE,     // EXE : check slave ausgefuehrt
  EXE_WRITE8_RAM,    // EXE : write ram ausgefuehrt (byte)
  EXE_READ8_RAM,     // EXE : read ram ausgefuehrt (byte)
  EXE_WRITE16_RAM,   // EXE : write ram ausgefuehrt (integer)
  EXE_READ16_RAM,    // EXE : read ram ausgefuehrt (integer)  
  ERR_NOT_DEFINED,   // ERR : error nr ist noch nicht definiert
  ERR_WRONG_XOR,     // ERR : empfangene Checksumme ist falsch
  ERR_WRONG_ID,      // ERR : empfangene ID ist falsch
  ERR_WRONG_POS,     // ERR : position der Bytes falsch
  ERR_WRONG_END,     // ERR : endekennung falsch
  ERR_UNKNOWN,       // ERR : unbekanntes Kommando empfangen
  ERR_ALL_DATA_0,    // ERR : Alle empfangene Daten sind 0
  ERR_ALL_DATA_1,    // ERR : Alle empfangene Daten sind 1
  ERR_WRONG_DATA     // ERR : ungueltige Daten empfangen  
}SPI_STATUS_ID_t;



//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void MK3_INIT_SPI_SLAVE(SPI_Slave_ID_t own_id);	// zum initialis. vom SPI als Slave
void MK3_SPIS_SET_STATUS(char status);          // setzt status byte
SPI_KOM_STATUS_t MK3_SPIS_AKTIVITY(void);       // erkennt SPI aktivitaeten
SPI_STATUS_ID_t MK3_SPIS_GET_ID(void);          // uebergibt letzte aktivitaet
void MK3_SPIS_WR8_RAM(char adr, char daten);    // schreibt ein byte ins RAM
unsigned char MK3_SPIS_RD8_RAM(char adr);       // liest ein byte aus dem RAM
void MK3_SPIS_WR16_RAM(char adr, unsigned int daten);  // schreibt ein INT ins RAM
unsigned int MK3_SPIS_RD16_RAM(char adr);       // liest ein INT aus dem RAM
//-----------------------------------------------------------------------------


//----------------------------------------------------------------------------
// Definition der SPI-Pins vom Slave
//----------------------------------------------------------------------------
#define MK3_DDR_SPI DDRB	// DDR-Register der SPI-Pins
#define MK3_PORT_SPI  PORTB     // Port der SPI-Pins

#define MK3_SPI_SS 0x01         // SS von SPI
#define MK3_SPI_SCK 0x02    	// SCK von SPI
#define MK3_SPI_MOSI 0x04       // MOSI von SPI
#define MK3_SPI_MISO 0x08       // MISO von SPI


//-----------------------------------------------------------------------------
// interne Funktionen vom Slave
//-----------------------------------------------------------------------------
void SPIS_make_ReadyCmd(void);
void SPIS_make_OkCmd(unsigned char cmd, unsigned char d0, unsigned char d1);
void SPIS_make_ErrCmd(SPI_B2_t wert);
void SPIS_make_UnknwonCmd(unsigned char cmd);
void SPIS_makeSlaveTx(char b0,char b1,char b2,char b3,char b4,char b5,char b6,char b7);
void SPIS_handle_RX_Command(SPI_B2_t err_status);
SPI_B2_t SPIS_check_Rx_Command(void);
SPI_B2_t SPIS_execute_RX_Command(void);


//-----------------------------------------------------------------------------
// interne Struktur vom Slave
//-----------------------------------------------------------------------------
typedef struct {
  unsigned char spi_pos[SPI_DATA_BYTES];  // byte positionen (vorberechnung)
  unsigned char ram[MAX_SPI_RAM];         // RAM fuer Datenaustausch mit Master
  unsigned char status;                   // status-byte (u.U. fuer Master)
  unsigned char err_cmd;                  // NACK kommando (vorberechnung)
  unsigned char old_cmd;                  // alter kommando zaehlerstand
  unsigned char akt_cmd;                  // aktuelle kommando zaehlerstand
  unsigned char sid;                      // Slave-ID
  SPI_STATUS_ID_t akt_status;             // aktueller status
  SPI_STATUS_ID_t last_status;            // letzter status
}SPI_SLAVE_t;
SPI_SLAVE_t SPI_SLAVE;


//----------------------------------------------------------------------
// Initialisiert die CPU als SPI-Slave
// mit einer bestimmten Slave-ID
// WICHTIG : die ID darf nur einmal im System benutzt werden
//----------------------------------------------------------------------
void MK3_INIT_SPI_SLAVE(SPI_Slave_ID_t own_id) {
  unsigned int n=0;
  unsigned char sid=0;
  
  #if SPI_DATA_BYTES != 10
    // WICHTIG !! Anzahl der DatenBytes MUSS 10 sein
    spi_forced_error=1; 
  #endif  
  
  // SPI-Pins definieren---------------------------
  MK3_DDR_SPI&=~MK3_SPI_SS;    // SS als Eingang
  MK3_PORT_SPI|=MK3_SPI_SS;    // Pull-Up aktivieren (WICHTIG)
  MK3_DDR_SPI&=~MK3_SPI_SCK;   // SCK als Eingang  
  MK3_DDR_SPI&=~MK3_SPI_MOSI;  // MOSI als Eingang
  MK3_DDR_SPI|=MK3_SPI_MISO;   // MISO als Ausgang
  
  // init der Struktur-------------------------------    
  for(n=0;n<SPI_DATA_BYTES;n++) {
    SPI_DATA.tx[n]=0x00; 
    SPI_DATA.rx[n]=0x00; 
  }
  
  // init vom RAM-------------------------------------
  for(n=0;n<MAX_SPI_RAM;n++) {
    SPI_SLAVE.ram[n]=0x00;
  }
  
  // MSB aller notwendigen RX-Bytes erstellen----------
  SPI_SLAVE.spi_pos[0]=(MSB_BYTE_0 & 0xF0);
  SPI_SLAVE.spi_pos[1]=(MSB_BYTE_1 & 0xF0);
  SPI_SLAVE.spi_pos[2]=(MSB_BYTE_2 & 0xF0);
  SPI_SLAVE.spi_pos[3]=(MSB_BYTE_3 & 0xF0);
  SPI_SLAVE.spi_pos[4]=(MSB_BYTE_4 & 0xF0);
  SPI_SLAVE.spi_pos[5]=(MSB_BYTE_5 & 0xF0);
  SPI_SLAVE.spi_pos[6]=(MSB_BYTE_6 & 0xF0);
  SPI_SLAVE.spi_pos[7]=(MSB_BYTE_7 & 0xF0); 
  SPI_SLAVE.spi_pos[8]=(MSB_BYTE_8 & 0xF0);
  SPI_SLAVE.spi_pos[9]=(MSB_BYTE_9 & 0xF0);
  
  // Slave ID speichern--------------------------------
  sid=SPIMS_getSID(own_id);
  SPI_SLAVE.sid=sid;   
  
  // error cmd erstellen
  SPI_SLAVE.err_cmd=SPIMS_makeByteLo(MSB_SLAVE_ID,CMD_NACK);      
  
  // Reset der Variablen------------------------------
  SPI_SLAVE.status=0x00;
  SPI_SLAVE.old_cmd=0;
  SPI_SLAVE.akt_cmd=0;
  SPI_SLAVE.akt_status=NO_SPI_KOM;
  SPI_SLAVE.last_status=NO_SPI_KOM;
    
  // erstes Sende Byte initialisieren-----------------
  SPIS_make_ReadyCmd();
  SPDR=SPI_DATA.tx[0];  

  // aktivieren des SPI-Ports--------------------------
  SPCR=0x00;
  SPCR|=0x80; // SPI_Interrupt aktivieren
  SPCR|=0x40; // SPI aktivieren  
  
  SREG|=0x80; // Alle Interrupts einschalten
 
}

//----------------------------------------------------------------------
// setzt das Status-Byte vom Slave
// auf einen Wert
// -> der Master kann diesen Wert mit "Check_Slave" auslesen
//----------------------------------------------------------------------
void MK3_SPIS_SET_STATUS(char status) {
  SPI_SLAVE.status=status;
}

//----------------------------------------------------------------------
// erkennt ob ein SPI Kommando empfangen wurde
// und ob es gueltig war und ausgefuehrt wurde
// oder ob es fehlerhaft war
//
// zum genaueren Unterscheiden dient danach "MK3_SPIS_GET_ID"
//----------------------------------------------------------------------
SPI_KOM_STATUS_t MK3_SPIS_AKTIVITY(void) {
  SPI_KOM_STATUS_t ret_wert=KOM_NOTHING;
  SPI_STATUS_ID_t check;

  check=SPI_SLAVE.last_status;
  if(check!=NO_SPI_KOM) {
    // falls eine aktivitaet aufgetreten ist

    // check ob error oder ok
    ret_wert=KOM_CMD_ERR;
    if(check==EXE_CHK_SLAVE) ret_wert=KOM_CMD_EXE;
    if(check==EXE_WRITE8_RAM) ret_wert=KOM_CMD_EXE;
    if(check==EXE_READ8_RAM) ret_wert=KOM_CMD_EXE;
    if(check==EXE_WRITE16_RAM) ret_wert=KOM_CMD_EXE;
    if(check==EXE_READ16_RAM) ret_wert=KOM_CMD_EXE;

    // aktivitaet speichern (zum spaeteren auslesen)
    SPI_SLAVE.akt_status=check;
    SPI_SLAVE.last_status=NO_SPI_KOM;
  }

  return(ret_wert);
}


//----------------------------------------------------------------------
// uebergibt die letzte Aktivitaets-ID
//----------------------------------------------------------------------
SPI_STATUS_ID_t MK3_SPIS_GET_ID(void) {
  SPI_STATUS_ID_t ret_wert;

  // aktuelle Kommunikations-ID auslesen
  ret_wert=SPI_SLAVE.akt_status;
  SPI_SLAVE.akt_status=NO_SPI_KOM;

  return(ret_wert);
}

//----------------------------------------------------------------------
// schreibt ein Byte ins RAM
//----------------------------------------------------------------------
void MK3_SPIS_WR8_RAM(char adr, char daten) {
  unsigned int n=0;

  n=adr;
  
  if(n<MAX_SPI_RAM) {
    SPI_SLAVE.ram[n]=daten;
  }
}

//----------------------------------------------------------------------
// liest ein Byte aus dem RAM
//----------------------------------------------------------------------
unsigned char MK3_SPIS_RD8_RAM(char adr) {
  unsigned char ret_wert=0;
  unsigned int n=0;

  n=adr;

  if(n<MAX_SPI_RAM) {
    ret_wert=SPI_SLAVE.ram[n];
  }
  
  return(ret_wert);
}

//----------------------------------------------------------------------
// schreibt ein INT ins RAM
//----------------------------------------------------------------------
void MK3_SPIS_WR16_RAM(char adr, unsigned int daten) {
  unsigned char lo,hi; 
  unsigned int n=0;
  
  n=adr; 
  
  if(n<MAX_SPI_RAM-1) {
    lo=(unsigned char)(daten&0x00FF);    
    hi=(unsigned char)(daten>>8);
    
    SPI_SLAVE.ram[n+0]=hi;
    SPI_SLAVE.ram[n+1]=lo;
  }
}

//----------------------------------------------------------------------
// liest ein INT aus dem RAM
//----------------------------------------------------------------------
unsigned int MK3_SPIS_RD16_RAM(char adr) {
  unsigned int ret_wert=0;
  unsigned char lo,hi;
  unsigned int n=0;

  n=adr;

  if(n<MAX_SPI_RAM-1) {
    hi=SPI_SLAVE.ram[n+0];
    lo=SPI_SLAVE.ram[n+1];
     
    ret_wert=(hi<<8);
    ret_wert=(ret_wert&0xFF00);
    ret_wert+=lo;
  }
  
  return(ret_wert);
}

//----------------------------------------------------------------------
// Hier werden gueltige Kommando vom Master ausgefuehrt
// und ein Antwort Kommando vorbereitet
//
// Vorsicht !! diese Funktion darf nicht zu langsam werden
// sie wird nach jedem empfangenen Kommando ausgefuehrt (nach 10 bytes)
// und muss so schnell wie moeglich wieder beendet werden
//
// im Moment (Version 1.8) dauert die Funktion ca. 70us @16MHz
//                                             ca. 150us @8MHz
//----------------------------------------------------------------------
SPI_B2_t SPIS_execute_RX_Command(void) {
  SPI_B2_t ret_wert=RX_OK;
  unsigned char cmd;
  unsigned char daten1,adr;
  unsigned char daten2;
    
  cmd=(SPI_DATA.rx[BPOS_CMD]&0x0F);
  // -------------------------------------------------------------------
  if(cmd==(CMD_CHK_SLAVE&0x0F)) {
    // -> Status vom Slave auslesen
    // OK und Statusbyte zurueckmelden
    SPIS_make_OkCmd(cmd,SPI_SLAVE.status,0x00);
    SPI_SLAVE.last_status=EXE_CHK_SLAVE;
  }
  // -------------------------------------------------------------------
  else if(cmd==(CMD_WR8_RAM&0x0F)) {
    // -> ein Byte ins RAM schreiben
    // Adresse = Byte 2+3
    // Daten = Byte 4+5
    adr=SPIMS_getByte(SPI_DATA.rx[2],SPI_DATA.rx[3]);  
    daten1=SPIMS_getByte(SPI_DATA.rx[4],SPI_DATA.rx[5]);  
    if(adr<MAX_SPI_RAM) {
      // daten ins ram speichern
      SPI_SLAVE.ram[adr]=daten1;
      // OK und daten zurueckmelden
      SPIS_make_OkCmd(cmd,daten1,0x00); 
      SPI_SLAVE.last_status=EXE_WRITE8_RAM;
    }
    else {
      // adresse nicht erlaubt
      SPIS_make_ErrCmd(RX_WRONG_DATA);
      ret_wert=RX_WRONG_DATA;
    }
  }
  // -------------------------------------------------------------------
  else if(cmd==(CMD_RD8_RAM&0x0F)) {
    // -> ein Byte aus dem RAM lesen
    // Adresse = Byte 2+3    
    adr=SPIMS_getByte(SPI_DATA.rx[2],SPI_DATA.rx[3]);      
    if(adr<MAX_SPI_RAM) {
      // daten aus ram lesen
      daten1=SPI_SLAVE.ram[adr];
      // OK und daten zurueckmelden
      SPIS_make_OkCmd(cmd,daten1,0x00); 
      SPI_SLAVE.last_status=EXE_READ8_RAM;      
    }
    else {
      // adresse nicht erlaubt
      SPIS_make_ErrCmd(RX_WRONG_DATA);
      ret_wert=RX_WRONG_DATA;
    }      
    
  }
  // -------------------------------------------------------------------
  else if(cmd==(CMD_WR16_RAM&0x0F)) {
    // -> ein Integer ins RAM schreiben
    // Adresse = Byte 2+3
    // Daten = Byte 4+5+6+7    
    adr=SPIMS_getByte(SPI_DATA.rx[2],SPI_DATA.rx[3]);  
    daten1=SPIMS_getByte(SPI_DATA.rx[4],SPI_DATA.rx[5]); 
    daten2=SPIMS_getByte(SPI_DATA.rx[6],SPI_DATA.rx[7]);
    if(adr<MAX_SPI_RAM-1) {
      // daten ins ram speichern
      SPI_SLAVE.ram[adr+0]=daten1;
      SPI_SLAVE.ram[adr+1]=daten2;
      // OK und daten zurueckmelden
      SPIS_make_OkCmd(cmd,daten1,daten2); 
      SPI_SLAVE.last_status=EXE_WRITE16_RAM;
    }
    else {
      // adresse nicht erlaubt
      SPIS_make_ErrCmd(RX_WRONG_DATA);
      ret_wert=RX_WRONG_DATA;
    }
  }  
  // -------------------------------------------------------------------
  else if(cmd==(CMD_RD16_RAM&0x0F)) {
    // -> ein Integer aus dem RAM lesen
    // Adresse = Byte 2+3    
    adr=SPIMS_getByte(SPI_DATA.rx[2],SPI_DATA.rx[3]);      
    if(adr<MAX_SPI_RAM-1) {
      // daten aus ram lesen
      daten1=SPI_SLAVE.ram[adr+0];
      daten2=SPI_SLAVE.ram[adr+1];
      // OK und daten zurueckmelden
      SPIS_make_OkCmd(cmd,daten1,daten2); 
      SPI_SLAVE.last_status=EXE_READ16_RAM;      
    }
    else {
      // adresse nicht erlaubt
      SPIS_make_ErrCmd(RX_WRONG_DATA);
      ret_wert=RX_WRONG_DATA;
    }      
    
  }  
  // -------------------------------------------------------------------
  else if(cmd==(CMD_POLL_SLAVE&0x0F)) {
    // -> gib die angeforderten Daten zurueck

    SPIS_make_ReadyCmd();  
  }
  // -------------------------------------------------------------------
  else {
    // -> Kommando ist dem Slave nicht bekannt

    SPIS_make_UnknwonCmd(cmd);
    ret_wert=RX_UNKNOWN;
  }
  
  return(ret_wert);
}

//----------------------------------------------------------------------
// Bearbeitet ein empfangenes Kommando vom Master
// wenn alle Daten plausibel und Fehlerfrei :
// -> ausfuehren des Kommandos
//----------------------------------------------------------------------
void SPIS_handle_RX_Command(SPI_B2_t err_status) {
  SPI_B2_t rx_err=RX_OK;
    
  rx_err=err_status;
  if(err_status==RX_OK) {
    // wenn kommando bis jetzt ok ist
    rx_err=SPIS_check_Rx_Command();
    if(rx_err==RX_OK) {
      // wenn die checksumme und die ID ok ist
      rx_err=SPIS_execute_RX_Command(); // kommando ausfuehren      
    }
    else {
      // wenn die checksumme oder ID falsch ist
      SPIS_make_ErrCmd(rx_err);       
    }
  }
  else {
    // RX-Kommando mit Fehler
    SPIS_make_ErrCmd(err_status);
  }
  
  //----------------------------------------
  // ergebnis auswerten
  //----------------------------------------
  if(rx_err!=RX_OK) {
    SPI_SLAVE.last_status=ERR_NOT_DEFINED;
    if(rx_err==RX_WRONG_XOR) SPI_SLAVE.last_status=ERR_WRONG_XOR;
    if(rx_err==RX_WRONG_ID) SPI_SLAVE.last_status=ERR_WRONG_ID;
    if(rx_err==RX_WRONG_POS) SPI_SLAVE.last_status=ERR_WRONG_POS;
    if(rx_err==RX_WRONG_END) SPI_SLAVE.last_status=ERR_WRONG_END;
    if(rx_err==RX_UNKNOWN) SPI_SLAVE.last_status=ERR_UNKNOWN;
    if(rx_err==RX_ALL_DATA_0) SPI_SLAVE.last_status=ERR_ALL_DATA_0;
    if(rx_err==RX_ALL_DATA_1) SPI_SLAVE.last_status=ERR_ALL_DATA_1;
    if(rx_err==RX_WRONG_DATA) SPI_SLAVE.last_status=ERR_WRONG_DATA;
  }  
}

//----------------------------------------------------------------------
// Erstellt ein Antwort-Komando fuer den Master
// -> Slave ist bereit fuer neue Daten
//----------------------------------------------------------------------
void SPIS_make_ReadyCmd(void) {  
  unsigned char sid=0;
  
  sid=SPI_SLAVE.sid;
  SPIS_makeSlaveTx(sid,CMD_READY,RX_WAIT4DATA,CMD_0,CMD_0,CMD_0,CMD_0,CMD_0);
}

//----------------------------------------------------------------------
// Erstellt ein Antwort-Komando fuer den Master
// -> empfangenes Kommando war ok und wurde ausgefuehrt
// -> die ausgefuehrte Kommand-Nr und zwei Daten Bytes
//    werden auch mit uebergeben
//----------------------------------------------------------------------
void SPIS_make_OkCmd(unsigned char cmd, unsigned char d0, unsigned char d1) {  
  unsigned char sid=0;
  unsigned char lo0,hi0;
  unsigned char lo1,hi1;  
  
  sid=SPI_SLAVE.sid;
  hi0=SPIMS_makeByteHi(0x00,d0);
  lo0=SPIMS_makeByteLo(0x00,d0); 
  hi1=SPIMS_makeByteHi(0x00,d1);
  lo1=SPIMS_makeByteLo(0x00,d1);   
  SPIS_makeSlaveTx(sid,CMD_ACK,hi0,lo0,hi1,lo1,CMD_0,CMD_0); 
}

//----------------------------------------------------------------------
// Erstellt ein Antwort-Komando fuer den Master
// -> empfangenes Kommando war Fehlerhaft
// -> eine Fehlernummer wird auch mit uebergeben
//----------------------------------------------------------------------
void SPIS_make_ErrCmd(SPI_B2_t wert) {  
  unsigned char sid=0;
  
  sid=SPI_SLAVE.sid;
  SPIS_makeSlaveTx(sid,CMD_NACK,wert,CMD_0,CMD_0,CMD_0,CMD_0,CMD_0);
}

//----------------------------------------------------------------------
// Erstellt ein Antwort-Komando fuer den Master
// -> empfangenes Kommando war unbekannt
// -> die empfangene Kommando-Nr wird auch mit uebergeben
//----------------------------------------------------------------------
void SPIS_make_UnknwonCmd(unsigned char cmd) {    
  unsigned char sid=0;
  
  sid=SPI_SLAVE.sid;
  SPIS_makeSlaveTx(sid,CMD_UNKNOWN,cmd,CMD_0,CMD_0,CMD_0,CMD_0,CMD_0);
}

//----------------------------------------------------------------------
// Erstellt ein Kommando (bestehend aus 10Bytes)
// das beim naechsten mal zum Master gesendet wird
// -> 8 Bytes werden uebergeben, 2Bytes bilden die Checksumme 
//----------------------------------------------------------------------
void SPIS_makeSlaveTx(char b0,char b1,char b2,char b3,char b4,char b5,char b6,char b7) {
  unsigned char chksum=0;
  
  // TX-Data erstellen  
  SPI_DATA.tx[0]=SPIMS_makeByteLo(MSB_SLAVE_ID,b0);
  SPI_DATA.tx[1]=SPIMS_makeByteLo(MSB_SLAVE_ID,b1);
  SPI_DATA.tx[2]=SPIMS_makeByteLo(MSB_SLAVE_ID,b2);
  SPI_DATA.tx[3]=SPIMS_makeByteLo(MSB_SLAVE_ID,b3);
  SPI_DATA.tx[4]=SPIMS_makeByteLo(MSB_SLAVE_ID,b4);
  SPI_DATA.tx[5]=SPIMS_makeByteLo(MSB_SLAVE_ID,b5);
  SPI_DATA.tx[6]=SPIMS_makeByteLo(MSB_SLAVE_ID,b6);
  SPI_DATA.tx[7]=SPIMS_makeByteLo(MSB_SLAVE_ID,b7);
  
  // checksumme erstellen
  chksum=SPIMS_makeChecksum_TX();
  
  // checksumme einfuegen
  SPI_DATA.tx[8]=SPIMS_makeByteHi(MSB_SLAVE_ID,chksum);
  SPI_DATA.tx[9]=SPIMS_makeByteLo(MSB_SLAVE_ID,chksum);  
 
  // letztes byte = startbyte
  SPI_DATA.tx[10]=SPI_DATA.tx[0];
}

//----------------------------------------------------------------------
// Ueberprufen des empfangenen Kommandos auf Fehler
// in der Checksumme oder in der ID
// erkennt auch ob alle Bits Lo oder alle Bits Hi sind
//----------------------------------------------------------------------
SPI_B2_t SPIS_check_Rx_Command(void) { 
  SPI_B2_t ret_wert=RX_OK;
  unsigned char wert1=0,wert2=0;
  unsigned int n,sum=0;
  
  // pruefen ob alle daten hi oder lo
  sum=0;
  for(n=0;n<SPI_DATA_BYTES;n++) {    
    sum+=SPI_DATA.rx[n];
  }
  if(sum==0) ret_wert=RX_ALL_DATA_0;
  if(sum==MAX_SPI_DATA_WERT) ret_wert=RX_ALL_DATA_1;   
  
  if(ret_wert==RX_OK) {
    // pruefen der empfangenen checksumme
    wert1=SPIMS_makeChecksum_RX();
    wert2=SPIMS_getByte(SPI_DATA.rx[BPOS_XOR_HI],SPI_DATA.rx[BPOS_XOR_LO]); 
    if(wert1!=wert2) {
      ret_wert=RX_WRONG_XOR; // falsche XOR
    }
    else {
      // pruefen auf die richtigen Slave-ID 
      wert1=(SPI_DATA.rx[BPOS_ID]&0x0F);
      wert2=(SPI_SLAVE.sid&0x0F);
      if(wert1!=wert2) ret_wert=RX_WRONG_ID; // falsche ID
    }
  }
  
  return(ret_wert);
}

//----------------------------------------------------------------------
// ISR : SPI-Byte empfangen
// wird pro empfangenem spi_byte aufgerufem
//----------------------------------------------------------------------
ISR (SPI_STC_vect) {                          // ISR : SPI
  unsigned char wert=0;
  unsigned char msb=0;
  static unsigned char akt_pos=0;  
  static unsigned char cnt_pos=0;
  static SPI_B2_t rx_err=RX_OK;
  unsigned char new_rx_data=0;
  
  // SPI Daten auslesen
  wert=SPDR; 
  msb=(wert&0xF0);
  
  cnt_pos++;
  
  if(rx_err==RX_OK) {
    // wenn noch kein fehler
    if(msb==SPI_SLAVE.spi_pos[akt_pos]) {
      // wenn die position stimmt
      // daten speichern
      SPI_DATA.rx[akt_pos]=wert;
      akt_pos++;
      // naechstes Byte vorbereiten
      new_rx_data=SPI_DATA.tx[akt_pos];      
    }
    else {
      // wenn die position nicht stimmt
      rx_err=RX_WRONG_POS;
      // naechstes Byte vorbereiten
      new_rx_data=SPI_SLAVE.err_cmd;      
    }
  } 
  else {
    // naechstes Byte vorbereiten
    new_rx_data=SPI_SLAVE.err_cmd;    
  }

  // warten auf letztes byte
  if((msb==SPI_SLAVE.spi_pos[SPI_DATA_BYTES-1]) || (cnt_pos>=SPI_DATA_BYTES)) {
    // wenn letztes zeichen empfangen wurde
    cnt_pos=0;       
    // pruefen ob richtige anzahl
    if(akt_pos!=SPI_DATA_BYTES) rx_err=RX_WRONG_END;     
    //------------------------------------------------------    
    // auswertung des empfangenen kommandos starten
    SPIS_handle_RX_Command(rx_err);        
    //------------------------------------------------------     
    SPI_SLAVE.akt_cmd++;
    akt_pos=0; 
    rx_err=RX_OK;
  }
  
  // neue Daten fuer den Master bereitstellen
  SPDR=new_rx_data;
}
